@@ -29,7 +29,7 @@ class glue_cross
     {
     static const bool is_row  = T1::is_row;
     static const bool is_col  = T1::is_col;
-    static const bool is_xvec = false;
+    static const bool is_xvec = true;
     };
   
   template<typename T1, typename T2> inline static void apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_cross>& X);
