@@ -301,6 +301,19 @@ struct is_Op< const Op<T1,op_type> >
   { static const bool value = true; };
 
 
+template<typename T>
+struct is_CubeToMatOp
+  { static const bool value = false; };
+ 
+template<typename T1, typename op_type>
+struct is_CubeToMatOp< CubeToMatOp<T1,op_type> >
+  { static const bool value = true; };
+ 
+template<typename T1, typename op_type>
+struct is_CubeToMatOp< const CubeToMatOp<T1,op_type> >
+  { static const bool value = true; };
+
+
 template<typename T>
 struct is_SpToDOp
   { static const bool value = false; };
@@ -383,6 +396,19 @@ struct is_mtGlue< const mtGlue<eT, T1, T2, glue_type> >
 //
 
 
+template<typename T>
+struct is_glue_times
+  { static const bool value = false; };
+
+template<typename T1, typename T2>
+struct is_glue_times< Glue<T1,T2,glue_times> >
+  { static const bool value = true; };
+
+template<typename T1, typename T2>
+struct is_glue_times< const Glue<T1,T2,glue_times> >
+  { static const bool value = true; };
+
+
 template<typename T>
 struct is_glue_times_diag
   { static const bool value = false; };
@@ -505,6 +531,7 @@ struct is_arma_type2
   =  is_Mat<T1>::value
   || is_Gen<T1>::value
   || is_Op<T1>::value
+  || is_CubeToMatOp<T1>::value
   || is_SpToDOp<T1>::value
   || is_Glue<T1>::value
   || is_eOp<T1>::value
@@ -605,6 +632,24 @@ struct is_SpSubview< SpSubview<eT> >
   { static const bool value = true; };
 
 
+template<typename T>
+struct is_SpSubview_col
+  { static const bool value = false; };
+
+template<typename eT>
+struct is_SpSubview_col< SpSubview_col<eT> >
+  { static const bool value = true; };
+
+
+template<typename T>
+struct is_SpSubview_row
+  { static const bool value = false; };
+
+template<typename eT>
+struct is_SpSubview_row< SpSubview_row<eT> >
+  { static const bool value = true; };
+
+
 template<typename T>
 struct is_spdiagview
   { static const bool value = false; };
@@ -657,6 +702,8 @@ struct is_arma_sparse_type
   static const bool value
   =  is_SpMat<T1>::value
   || is_SpSubview<T1>::value
+  || is_SpSubview_col<T1>::value
+  || is_SpSubview_row<T1>::value
   || is_spdiagview<T1>::value
   || is_SpOp<T1>::value
   || is_SpGlue<T1>::value
